\JPR.XPL	11-Nov-2017	Version 1.4	loren.blaney@gmail.com
\Jet Printer Utility
\Configures HP LaserJet, Inkjet or DeskJet printer and prints the text file
\ entered on the command line. Lines greater than the width of the page are
\ clipped off. This saves the configuration information in a file called JPR.INI
\ in the root directory of the C: drive.
\
\REVISIONS:
\ 17-Jan-2006: Original release.
\ 12-Mar-2011: Don't print anything if number of copies = 0.
\ 11-Nov-2017: LF = newline, for Linux compatibility.

include c:\cxpl\codesi;	\include intrinsic code declarations

int	PaperSize,	\(11 or 14 inches)
	Orientation,	\0=portrait or 1=landscape
	LinesPerPage,	\maximum number of lines per page
	CharsPerInch,	\horizontal character spacing
	StrokeWeight,	\character boldness (-7..+7)
	MarginSize,	\left margin size (characters)
	Copies;		\number of copies to print (defaults to 1)

int	CpuReg,		\CPU register array
	DataSeg,	\segment address of our data
	PspSeg,		\segment address of our Program Segment Prefix
	Handle;		\input file's handle

def	Tab=$09, LF=$0A, FF=$0C, CR=$0D, EOF=$1A, Esc=$1B;



proc	EscTxt(Cmd);	\Send escape command to printer
char	Cmd;
begin
ChOut(5, Esc);
Text(5, Cmd);
end;	\EscTxt



proc	SetConfig;	\Set configuration in printer
real	VMI;		\vertical motion index (1/48ths inch)
begin
EscTxt("E");		\reset printer
EscTxt("(10U");		\PC-8 symbol set (IBM's extended character set--"OEM")

\Set paper size (letter or legal):
EscTxt("&l"); ChOut(5, if PaperSize=14 then ^3 else ^2); ChOut(5, ^A);

\Set orientation (0=portrait, 1=landscape):
EscTxt("&l"); IntOut(5, Orientation); ChOut(5, ^O);

\Set lines per page:
VMI:= Float(48*(PaperSize-1)) / Float(LinesPerPage);
if Orientation # 0 then		\landscape
	VMI:= 48.0*(8.5-1.0) / Float(LinesPerPage);
Format(5, 4);
EscTxt("&l"); RlOut(5, VMI); ChOut(5, ^C);
EscTxt("&l"); IntOut(5, LinesPerPage); ChOut(5, ^F);

\Set characters per inch:
EscTxt("(s"); IntOut(5, CharsPerInch); ChOut(5, ^H);

\Set stroke weight:
EscTxt("(s"); IntOut(5, StrokeWeight); ChOut(5, ^B);

\Set left margin:
EscTxt("&a"); IntOut(5, MarginSize); ChOut(5, ^L);
end;	\SetConfig



func	KeyLook;	\Get a keystroke without removing it from the buffer
begin
repeat	CpuReg(0):= $0100;	\get keyboard status
	SoftInt($16);		\call BIOS
until (CpuReg(16) & $40) = 0;	\wait until Z flag is clear (0)
return CpuReg(0) & $00FF;	\then return the character (ASCII)
end;	\KeyLook



proc	Adjust(Str, AddrVal);	\Display value and optionally change it
int	Str, AddrVal;
begin
Text(0, Str);
IntOut(0, AddrVal(0));		\show value
ChOut(0, Tab);
OpenI(0);			\(beware of entering decimal point)
if KeyLook # CR then		\if not carriage return then
	AddrVal(0):= IntIn(0)	\ get new value
else	ChIn(0);		\else handle the CR
end;	\Adjust



proc	GetConfig;	\Get configuration settings from keyboard
begin
Adjust("Page size (inches):       ", addr PaperSize);
Adjust("0=portrait, 1=landscape:  ", addr Orientation);
Adjust("Lines per page:           ", addr LinesPerPage);
Adjust("Characters per inch:      ", addr CharsPerInch);
Adjust("Stroke weight (-7..+7):   ", addr StrokeWeight);
Adjust("Left margin (characters): ", addr MarginSize);
Adjust("Number of copies:         ", addr Copies);
end;	\GetConfig



proc	SaveConfig;	\Save configuration data
int	Hand;
begin
Trap(false);				\don't abort if an error is detected
Hand:= FOpen("C:\JPR.INI", 1);		\get output file handle
Trap(true);
if GetErr \#0\ then []			\if error detected then ignore request
else	begin
	FSet(Hand, ^O);
	OpenO(3);
	IntOut(3, PaperSize); CrLf(3);
	IntOut(3, Orientation); CrLf(3);
	IntOut(3, LinesPerPage); CrLf(3);
	IntOut(3, CharsPerInch); CrLf(3);
	IntOut(3, StrokeWeight); CrLf(3);
	IntOut(3, MarginSize); CrLf(3);
	Close(3);
	FClose(Hand);
	end;
end;	\SaveConfig



proc	LoadConfig;	\Load configuration file (or load default values)
int	Hand;		\file handle
begin
Trap(false);				\don't trap "file not found" error
Hand:= FOpen("C:\JPR.INI", 0);		\open file for input
Trap(true);
if GetErr \#0\ then			\if configuration file is not found
	begin				\ then use default values
	PaperSize:= 11;
	Orientation:= 0;
	LinesPerPage:= 60;
	CharsPerInch:= 10;
	StrokeWeight:= 0;
	MarginSize:= 5;
	end
else	begin
	FSet(Hand, ^I);			\set device 3 to handle
	OpenI(3);
	PaperSize:= IntIn(3);
	Orientation:= IntIn(3);
	LinesPerPage:= IntIn(3);
	CharsPerInch:= IntIn(3);
	StrokeWeight:= IntIn(3);
	MarginSize:= IntIn(3);
	FClose(Hand);
	end;
Copies:= 1;
end;	\LoadConfig



func	OpenFile;	\Open the file specified on the command line
			\Returns 'false' if file is not found
char	FN($7F);	\file name
begin
Blit(PspSeg, $81, DataSeg, FN, $7F);	\copy command line into FN array
Trap(false);
Handle:= FOpen(FN, 0);			\get input handle
Trap(true);
if GetErr \#0\ then
	begin
	Text(0, "File not found"); CrLf(0);
	return false;
	end;
return true;
end;	\OpenFile



proc	PrintFile;	\Print the file specified on the command line
int	I, Ch;
begin
Text(0, "Printing...");
FSet(Handle, ^I);
for I:= 0, Copies-1 do
	begin
	OpenI(3);
	loop	begin
		Ch:= ChIn(3);
		if Ch = LF then ChOut(5, CR);
		if Ch = EOF then quit;
		ChOut(5, Ch);
		end;
	ChOut(5, FF);
	end;
FClose(Handle);
CrLf(0);
end;	\PrintFile



begin	\Main
CpuReg:= GetReg;
PspSeg:= CpuReg(11);
DataSeg:= CpuReg(12);

Text(0, "LaserJet, Inkjet and DeskJet Printing Utility, V1.4

");
if not OpenFile then exit 1;
LoadConfig;
GetConfig;
SaveConfig;
if Copies > 0 then
	begin
	SetConfig;
	PrintFile;
	EscTxt("E");		\reset printer
	end;
end;	\Main
